/*************************************************************************** 
* This file is part of the Frescobaldi project, http://www.frescobaldi.org/
*
* Copyright (c) 2008 - 2012 by Wilbert Berendsen
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
* See http://www.gnu.org/licenses/ for more information.
*/

function hover(){
    var url = this.getAttribute('xlink:href');
    pyLinks.hover(url);	
};

function leave(){
    var url = this.getAttribute('xlink:href');
    pyLinks.leave(url);	
};

function click(){
    var url = this.getAttribute('xlink:href');
    pyLinks.click(url);
};

var a = document.getElementsByTagName('a');
//loop through all links
for (var i= 0; i < a.length; ++i){

	a[i].onmouseover = hover;
    a[i].onmouseout = leave;
	a[i].onclick = click;
}
	


